package com.virjar.g4proxy;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Handler;
import android.os.IBinder;
import android.os.Looper;
import android.os.Message;
import android.os.SystemClock;
import android.util.Log;
import android.widget.Toast;

import com.virjar.g4proxy.utils.HttpClientUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;

import javax.annotation.Nullable;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.Request;
import okhttp3.Response;
import top.wuhaojie.installerlibrary.AutoInstaller;

public class AppUpdateService extends Service {

    //查询更新的间隔 （单位：分钟）
    public static final int updateInterval = 10;

    public static final String TAG = "AppUpdateService";

    public static final String getNewestModifiedApkUrl = "http://www.scumall.com:5597/hermes/ModifiedApp/getNewestModifiedApk?appPackageName=";


    @Nullable
    @Override
    public IBinder onBind(Intent intent) {

        startService();
        return null;
    }

    @Override
    public void onCreate() {

        super.onCreate();
        startService();
    }


    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {

        startService();
        return START_STICKY;
    }


    private void startService() {

        final String packageName = getApplicationInfo().packageName;
        Request request = HttpClientUtils.getRequest(getNewestModifiedApkUrl + packageName);

        HttpClientUtils.getClient().newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(@Nullable Call call, @Nullable IOException e) {
                Log.e("get request", "get downloadUrl failed", e);
            }

            @Override
            public void onResponse(@Nullable Call call, @Nullable Response response) {
                try {
                    handleResponse(response, packageName);
                } catch (JSONException | PackageManager.NameNotFoundException | IOException e) {
                    e.printStackTrace();
                }
            }
        });

        AlarmManager alarmManager = (AlarmManager) getSystemService(Context.ALARM_SERVICE);
        int time = 1000 * 60 * updateInterval;
        //获取系统开机至今所经历的毫秒
        long startTime = SystemClock.elapsedRealtime() + time;
        Intent intent = new Intent(this, AppUpdateService.class);
        PendingIntent pendingIntent = PendingIntent.getService(this, 0, intent, 0);
        if (alarmManager == null){
            return;
        }
        alarmManager.set(AlarmManager.ELAPSED_REALTIME_WAKEUP, startTime, pendingIntent);
    }

    private void doInstall(final String url) {
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                Log.e(TAG, "installing");
                AutoInstaller installer = AutoInstaller.getDefault(AppUpdateService.this);
                installer.installFromUrl(url);
                installer.setOnStateChangedListener(new AutoInstaller.OnStateChangedListener() {
                    @Override
                    public void onStart() {

                    }

                    @Override
                    public void onComplete() {

                    }

                    @Override
                    public void onNeed2OpenService() {
                        Toast.makeText(AppUpdateService.this, "请打开辅助功能服务", Toast.LENGTH_SHORT).show();
                    }

                    @Override
                    public void needPermission() {
                        Toast.makeText(AppUpdateService.this, "需要申请存储空间权限", Toast.LENGTH_SHORT).show();
                    }
                });
            }
        });
    }

    public void handleResponse(Response response, String packageName) throws JSONException, PackageManager.NameNotFoundException, IOException {

        if (response.body() == null){
            return;
        }
        String responseJson = response.body().string();
        JSONObject jsonObject = new JSONObject(responseJson);
        jsonObject = jsonObject.optJSONObject("data");
        String downloadUrl = jsonObject.optString("downloadUrl");
        long versionCode = jsonObject.optLong("versionCode");

        Log.i("app_update", "downloadUrl: " + downloadUrl);
        Log.i("app_update", "up-to-date versionCode: " + versionCode);
        PackageInfo packageInfo = getPackageManager().getPackageInfo(packageName, PackageManager.GET_META_DATA);
        if (packageInfo.versionCode < versionCode) {
            doInstall(downloadUrl);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }
}
